# ================================================================
# Name:      KeyGeneratorDialog.py
# Purpose:   Key generator analyzer dialogue.
# Author:    Joe Linoff
# Created:   2012-01-10
# Id:        $Id: KeyGeneratorDialog.py,v 1.2 2012/02/04 03:52:27 jlinoff Exp jlinoff $
# Copyright: (c) 2012 Joe Linoff
# License:
#     SecEd is free software; you can redistribute 
#     it and/or modify it under the terms of the GNU General Public License as 
#     published by the Free Software Foundation; either version 2 of the License, 
#     or (at your option) any later version.
#            
#     SecEd is distributed in the hope that it will be useful, 
#     but WITHOUT ANY WARRANTY; without even the implied warranty of 
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
#     See the GNU General Public License for more details. You should have 
#     received a copy of the GNU General Public License along with SecEd; 
#     if not, write to the Free Software Foundation, Inc., 59 Temple Place, 
#     Suite 330, Boston, MA  02111-1307  USA
# ================================================================

import string
import re
import sys
import os
import wx
import KeyGenerator
from DisplayStaticTextDialog import DisplayStaticTextDialog
_ = wx.GetTranslation

class KeyGeneratorDialog(wx.Dialog):
    """
    Determine the generator of a user key.
    """

    def __init__(self, parent, id, title, size):
        style = wx.DEFAULT_DIALOG_STYLE | wx.RESIZE_BORDER
        #style = wx.DEFAULT_DIALOG_STYLE
        wx.Dialog.__init__(self, parent, id, title, size=size, style=style)

        # The layout elements.
        vsizer  = wx.BoxSizer(wx.VERTICAL)
        hsizer1 = wx.BoxSizer(wx.HORIZONTAL)
        hsizer2 = wx.BoxSizer(wx.HORIZONTAL)
        hsizer3 = wx.BoxSizer(wx.HORIZONTAL)
        hsizer4 = wx.BoxSizer(wx.HORIZONTAL)
        hsizer5 = wx.BoxSizer(wx.HORIZONTAL)
        hsizer6 = wx.BoxSizer(wx.HORIZONTAL)

        self.BORDER=5

        # Fonts
        font10 = wx.Font(10, wx.DEFAULT, wx.NORMAL, wx.NORMAL, 0)
        font12 = wx.Font(12, wx.DEFAULT, wx.NORMAL, wx.NORMAL, 0)
        font14 = wx.Font(14, wx.DEFAULT, wx.NORMAL, wx.NORMAL, 0)
        font_text = wx.Font(12, wx.DEFAULT, wx.NORMAL, wx.NORMAL, 0, 'Courier')
        fontdef = font10

        # Key length.
        self._keyLenDefault = 8
        self._keyLen = [None, None]
        self._keyLen[0] = wx.StaticText(self,-1,_("Key Length:"),style=wx.ALIGN_RIGHT)
        self._keyLen[0].SetFont(fontdef)

        self._keyLen[1] = wx.TextCtrl(self, -1, str(self._keyLenDefault), size=(100,-1))
        self._keyLen[1].Bind(wx.EVT_KEY_UP, self.OnKeyLen)
        self._keyLen[1].SetFont(fontdef)

        hsizer1.Add(self._keyLen[0], 0, wx.ALIGN_CENTER_VERTICAL|wx.LEFT|wx.EXPAND, self.BORDER)
        hsizer1.Add(self._keyLen[1], 0, wx.ALIGN_CENTER_VERTICAL|wx.LEFT|wx.EXPAND, self.BORDER)

        # Number of keys.
        self._numKeys = [None, None]
        self._numKeys[0] = wx.StaticText(self,-1,_("Num Keys:"),style=wx.ALIGN_RIGHT)
        self._numKeys[0].SetFont(fontdef)

        self._numKeys[1] = wx.TextCtrl(self, -1, "5", size=(100,-1))
        self._numKeys[1].Bind(wx.EVT_KEY_UP, self.OnNumKeys)
        self._numKeys[1].SetFont(fontdef)

        hsizer2.Add(self._numKeys[0], 0, wx.ALIGN_CENTER_VERTICAL|wx.LEFT|wx.EXPAND, self.BORDER)
        hsizer2.Add(self._numKeys[1], 0, wx.ALIGN_CENTER_VERTICAL|wx.LEFT|wx.EXPAND, self.BORDER)

        # Charsets
        sbox = wx.StaticBox(self, wx.ID_ANY, '')
        sbox.SetFont(fontdef)
        sboxsizer = wx.StaticBoxSizer(sbox, wx.VERTICAL)

        choices = []
        for i in range(self._keyLenDefault+1):
            choices.append(str(i))

        # Titles
        self._title = [None, None, None, None]

        self._title[0] = wx.StaticText(self, 1, _('Charset'), style=wx.ALIGN_LEFT)
        self._title[0].SetFont(fontdef)

        self._title[1] = wx.StaticText(self, 1, _('Minimum'), style=wx.ALIGN_LEFT)
        self._title[1].SetFont(fontdef)

        self._title[2] = wx.StaticText(self, 1, _('Maximum'), style=wx.ALIGN_LEFT)
        self._title[2].SetFont(fontdef)

        self._title[3] = wx.StaticText(self, 1, _('Duplicates'), style=wx.ALIGN_LEFT)
        self._title[3].SetFont(fontdef)

        # Lower case letters.
        half = self._keyLenDefault / 2
        self._alphaLC = [None, None, None, None]

        self._alphaLC[0] = wx.CheckBox(self,-1,'Lower Case')
        self._alphaLC[0].SetFont(fontdef)
        self._alphaLC[0].SetValue(True)
        self.Bind(wx.EVT_CHECKBOX, self.OnCbAlphaLC, self._alphaLC[0])

        self._alphaLC[1] = wx.ComboBox(self,-1,choices=choices, value=str(half), style=wx.CB_READONLY)
        self._alphaLC[1].SetFont(fontdef)
        self.Bind(wx.EVT_COMBOBOX, self.OnCbAlphaLCMin, self._alphaLC[1])

        self._alphaLC[2] = wx.ComboBox(self,-1,choices=choices, value=str(half+1), style=wx.CB_READONLY)
        self._alphaLC[2].SetFont(fontdef)
        self.Bind(wx.EVT_COMBOBOX, self.OnCbAlphaLCMax, self._alphaLC[2])

        self._alphaLC[3] = wx.CheckBox(self,-1,'')
        self._alphaLC[3].SetFont(fontdef)
        self._alphaLC[3].SetValue(False)
        self.Bind(wx.EVT_CHECKBOX, self.OnCbAlphaLCDup, self._alphaLC[3])

        # Upper case letters.
        self._alphaUC = [None, None, None, None]

        self._alphaUC[0] = wx.CheckBox(self,-1,'Upper Case')
        self._alphaUC[0].SetFont(fontdef)
        self._alphaUC[0].SetValue(True)
        self.Bind(wx.EVT_CHECKBOX, self.OnCbAlphaUC, self._alphaUC[0])

        self._alphaUC[1] = wx.ComboBox(self,-1,choices=choices, value='1', style=wx.CB_READONLY)
        self._alphaUC[1].SetFont(fontdef)
        self.Bind(wx.EVT_COMBOBOX, self.OnCbAlphaUCMin, self._alphaUC[1])

        self._alphaUC[2] = wx.ComboBox(self,-1,choices=choices, value='1', style=wx.CB_READONLY)
        self._alphaUC[2].SetFont(fontdef)
        self.Bind(wx.EVT_COMBOBOX, self.OnCbAlphaUCMax, self._alphaUC[2])

        self._alphaUC[3] = wx.CheckBox(self,-1,'')
        self._alphaUC[3].SetFont(fontdef)
        self._alphaUC[3].SetValue(False)
        self.Bind(wx.EVT_CHECKBOX, self.OnCbAlphaUCDup, self._alphaUC[3])

        # Digits
        self._digits = [None, None, None, None]

        self._digits[0] = wx.CheckBox(self,-1,'Digits')
        self._digits[0].SetFont(fontdef)
        self._digits[0].SetValue(True)
        self.Bind(wx.EVT_CHECKBOX, self.OnCbDigits, self._digits[0])

        self._digits[1] = wx.ComboBox(self,-1,choices=choices, value='1', style=wx.CB_READONLY)
        self._digits[1].SetFont(fontdef)
        self.Bind(wx.EVT_COMBOBOX, self.OnCbDigitsMin, self._digits[1])

        self._digits[2] = wx.ComboBox(self,-1,choices=choices, value='1', style=wx.CB_READONLY)
        self._digits[2].SetFont(fontdef)
        self.Bind(wx.EVT_COMBOBOX, self.OnCbDigitsMax, self._digits[2])

        self._digits[3] = wx.CheckBox(self,-1,'')
        self._digits[3].SetFont(fontdef)
        self._digits[3].SetValue(False)
        self.Bind(wx.EVT_CHECKBOX, self.OnCbDigitsDup, self._digits[3])

        # Spaces
        self._spaces = [None, None, None, None]

        self._spaces[0] = wx.CheckBox(self,-1,'Spaces')
        self._spaces[0].SetFont(fontdef)
        self._spaces[0].SetValue(True)
        self.Bind(wx.EVT_CHECKBOX, self.OnCbSpaces, self._spaces[0])

        self._spaces[1] = wx.ComboBox(self,-1,choices=choices, value='0', style=wx.CB_READONLY)
        self._spaces[1].SetFont(fontdef)
        self.Bind(wx.EVT_COMBOBOX, self.OnCbSpacesMin, self._spaces[1])

        self._spaces[2] = wx.ComboBox(self,-1,choices=choices, value='1', style=wx.CB_READONLY)
        self._spaces[2].SetFont(fontdef)
        self.Bind(wx.EVT_COMBOBOX, self.OnCbSpacesMax, self._spaces[2])

        self._spaces[3] = wx.CheckBox(self,-1,'')
        self._spaces[3].SetFont(fontdef)
        self._spaces[3].SetValue(False)
        self.Bind(wx.EVT_CHECKBOX, self.OnCbSpacesDup, self._spaces[3])

        # Other
        self._other = [None, None, None, None]

        self._other[0] = wx.CheckBox(self,-1,'Other')
        self._other[0].SetFont(fontdef)
        self._other[0].SetValue(True)
        self.Bind(wx.EVT_CHECKBOX, self.OnCbOther, self._other[0])

        self._other[1] = wx.ComboBox(self,-1,choices=choices, value='1', style=wx.CB_READONLY)
        self._other[1].SetFont(fontdef)
        self.Bind(wx.EVT_COMBOBOX, self.OnCbOtherMin, self._other[1])

        self._other[2] = wx.ComboBox(self,-1,choices=choices, value='1', style=wx.CB_READONLY)
        self._other[2].SetFont(fontdef)
        self.Bind(wx.EVT_COMBOBOX, self.OnCbOtherMax, self._other[2])

        self._other[3] = wx.CheckBox(self,-1,'')
        self._other[3].SetFont(fontdef)
        self._other[3].SetValue(False)
        self.Bind(wx.EVT_CHECKBOX, self.OnCbOtherDup, self._other[3])

        # Layout
        gs = wx.GridSizer(5,4,5,5)
        gs.AddMany( [

                (self._title[0], 0, wx.EXPAND),
                (self._title[1], 0, wx.EXPAND),
                (self._title[2], 0, wx.EXPAND),
                (self._title[3], 0, wx.EXPAND),

                (self._alphaLC[0], 0),
                (self._alphaLC[1], 0),
                (self._alphaLC[2], 0),
                (self._alphaLC[3], 0),

                (self._alphaUC[0], 0),
                (self._alphaUC[1], 0),
                (self._alphaUC[2], 0),
                (self._alphaUC[3], 0),

                (self._digits[0], 0),
                (self._digits[1], 0),
                (self._digits[2], 0),
                (self._digits[3], 0),

                (self._spaces[0], 0),
                (self._spaces[1], 0),
                (self._spaces[2], 0),
                (self._spaces[3], 0),

                (self._other[0], 0),
                (self._other[1], 0),
                (self._other[2], 0),
                (self._other[3], 0),

                ] )
        
        sboxsizer.Add(gs)

        # Exclude strings
        self._exclude = [None, None]
        self._exclude[0] = wx.StaticText(self,-1,_("Exclude:"),style=wx.ALIGN_RIGHT)
        self._exclude[0].SetFont(fontdef)

        self._exclude[1] = wx.TextCtrl(self, -1, _('Ol'), size=(400,-1))
        self._exclude[1].Bind(wx.EVT_KEY_UP, self.OnKeyLen)
        self._exclude[1].SetFont(fontdef)

        h = wx.BoxSizer(wx.HORIZONTAL)
        h.Add(self._exclude[0], 0, wx.ALIGN_CENTER_VERTICAL|wx.TOP|wx.LEFT|wx.EXPAND, self.BORDER)
        h.Add(self._exclude[1], 0, wx.ALIGN_CENTER_VERTICAL|wx.TOP|wx.LEFT|wx.EXPAND, self.BORDER)
        
        sboxsizer.Add(h)

        # Custom strings
        self._custom = [None, None]
        self._custom[0] = wx.StaticText(self,-1,_("Custom:"),style=wx.ALIGN_RIGHT)
        self._custom[0].SetFont(fontdef)

        self._custom[1] = wx.TextCtrl(self, -1, _(''), size=(400,-1))
        self._custom[1].Bind(wx.EVT_KEY_UP, self.OnKeyLen)
        self._custom[1].SetFont(fontdef)

        h = wx.BoxSizer(wx.HORIZONTAL)
        h.Add(self._custom[0], 0, wx.ALIGN_CENTER_VERTICAL|wx.TOP|wx.LEFT|wx.EXPAND, self.BORDER)
        h.Add(self._custom[1], 0, wx.ALIGN_CENTER_VERTICAL|wx.TOP|wx.LEFT|wx.EXPAND, self.BORDER)
        
        sboxsizer.Add(h)

        hsizer3.Add(sboxsizer, 0)

        # Status area
        self._text = wx.TextCtrl(self, -1, style=wx.ALIGN_LEFT|wx.TE_MULTILINE|wx.TE_READONLY)
        self._text.SetFont(font_text)
        self._text.SetInitialSize((750, 200))
        hsizer4.Add(self._text, 1, wx.ALIGN_CENTER_VERTICAL|wx.LEFT|wx.RIGHT|wx.BOTTOM|wx.EXPAND|wx.ALL, self.BORDER)

        # Buttons
        okBtn     = wx.Button(self, wx.ID_ANY, 'OK')
        doneBtn   = wx.Button(self, wx.ID_ANY, 'Done')
        helpBtn   = wx.Button(self, wx.ID_ANY, 'Help')

        hsizer5.Add(okBtn   , 0, wx.ALL, self.BORDER)
        hsizer5.Add(doneBtn , 0, wx.ALL, self.BORDER)
        hsizer5.Add(helpBtn , 0, wx.ALL, self.BORDER)

        self.Bind(wx.EVT_BUTTON, self.OnOk   , okBtn)
        self.Bind(wx.EVT_BUTTON, self.OnDone , doneBtn)
        self.Bind(wx.EVT_BUTTON, self.OnHelp , helpBtn)

        # Layout
        vsizer.Add(hsizer1, 0, wx.LEFT|wx.RIGHT|wx.BOTTOM|wx.TOP|wx.EXPAND, 2*self.BORDER)
        vsizer.Add(hsizer2, 0, wx.LEFT|wx.RIGHT|wx.BOTTOM|wx.EXPAND, 2*self.BORDER)
        vsizer.Add(hsizer3, 0, wx.LEFT|wx.RIGHT|wx.BOTTOM|wx.EXPAND, 2*self.BORDER)
        vsizer.Add(hsizer4, 1, wx.LEFT|wx.RIGHT|wx.BOTTOM|wx.EXPAND, 2*self.BORDER)
        vsizer.Add(hsizer5, 0, wx.LEFT|wx.RIGHT|wx.BOTTOM|wx.EXPAND, 2*self.BORDER)

        self.Layout()
        self.SetSizer(vsizer)
        self.Fit()

        # Now set the minimum based on the fit operation.
        size = self.GetSize()
        self.SetSizeHints( minW=size.GetWidth(), minH=size.GetHeight() )

    def AppendCharset(self, charsets, char_set, minval, maxval, dups):
        try:
            arg = '['+char_set+']'
            if minval:
                arg += ',min-matches='+minval
            if maxval:
                arg += ',max-matches='+maxval
            if dups:
                arg += ',max-num-dups='+maxval+',max-dup-size=2'
            charset = KeyGenerator.CharSet(arg).Init()
            charsets.append( charset )
        except Exception as (strerr):
            self.MyWarning('Character set failed to compile: %s: %s' % (arg, strerr))

    def OnOk(self, e):
        if e:
            e.Skip()

        # Step #1: make sure that the minimums are less than or equal to the length.
        keylen = int(self._keyLen[1].GetValue())
        minlen = 0
        maxlen = 0
        if keylen<3:
            self.MyWarning('Length is too short, minimum length is 3')
            self._keyLen[1].SetValue('3')
            return

        if self._alphaLC[0].GetValue():
            minlen += int(self._alphaLC[1].GetValue());
            maxlen += int(self._alphaLC[2].GetValue());

        if self._alphaUC[0].GetValue():
            minlen += int(self._alphaUC[1].GetValue());
            maxlen += int(self._alphaUC[2].GetValue());

        if self._digits[0].GetValue():
            minlen += int(self._digits[1].GetValue());
            maxlen += int(self._digits[2].GetValue());

        if self._spaces[0].GetValue():
            minlen += int(self._spaces[1].GetValue());
            maxlen += int(self._spaces[2].GetValue());

        if self._other[0].GetValue():
            minlen += int(self._other[1].GetValue());
            maxlen += int(self._other[2].GetValue());

        text = ''
        self._text.SetValue(text.rstrip())
        self.Refresh()
        
        charsets = []
        if minlen>keylen:
            self.MyWarning('The minimum length of all of the charsets (%d) exceeds the length specified for the key (%d). Cannot run the key generation algorithm because it is impossible to meet the acceptance criteria.' % (minlen,keylen))
        elif maxlen<keylen:
            self.MyWarning('The maximum length of all of the charsets (%d) is less than the length specified for the key (%d). Cannot run the key generation algorithm because it is impossible to meet the acceptance criteria.' % (minlen,keylen))
        else:
            # Generate the charsets.
            if self._alphaLC[0].GetValue():
                minval = self._alphaLC[1].GetValue()
                maxval = self._alphaLC[2].GetValue()
                dups   = self._alphaLC[3].GetValue()
                self.AppendCharset(charsets,'a-z',minval,maxval,dups)

            if self._alphaUC[0].GetValue():
                minval = self._alphaUC[1].GetValue()
                maxval = self._alphaUC[2].GetValue()
                dups   = self._alphaUC[3].GetValue()
                self.AppendCharset(charsets,'A-Z',minval,maxval,dups)

            if self._digits[0].GetValue():
                minval = self._digits[1].GetValue()
                maxval = self._digits[2].GetValue()
                dups   = self._digits[3].GetValue()
                self.AppendCharset(charsets,'0-9',minval,maxval,dups)

            if self._spaces[0].GetValue():
                minval = self._spaces[1].GetValue()
                maxval = self._spaces[2].GetValue()
                dups   = self._spaces[3].GetValue()
                self.AppendCharset(charsets,' ',minval,maxval,dups)

            if self._other[0].GetValue():
                minval = self._other[1].GetValue()
                maxval = self._other[2].GetValue()
                dups   = self._other[3].GetValue()
                self.AppendCharset(charsets,'-&%,!@();.+_[]^~',minval,maxval,dups)

            arg = self._exclude[1].GetValue()
            if arg not in ['',None]:
                minval = '0'
                maxval = None
                self.AppendCharset(charsets,arg,minval,maxval,False)

        try:
            arg = self._custom[1].GetValue()
            if arg not in ['',None]:
                charset = KeyGenerator.CharSet(arg,True).Init()
                charsets.append( charset )
        except Exception as (strerr):
            self.MyWarning('Custom command failed to compile: %s: %s' % (arg, strerr))

        if len(charsets)>0:
            num = int(self._numKeys[1].GetValue())
            for i in range(num):
                passwd = KeyGenerator.KeyGenerator(charsets,keylen)
                text += passwd + '\n'

            self._text.SetValue(text.rstrip())
            self.Refresh()

    def MyPopUp(self, title, msg):
        """
        Pops up a dialogue with a message.
        Used for internal debugging.
        """
        topWin = self
        dlg = wx.MessageDialog(topWin,msg,title,wx.OK)
        dlg.ShowModal()
        dlg.Destroy()

    def MyWarning(self, msg):
        self.MyPopUp('WARNING','WARNING! '+msg)

    def OnNumKeys(self, e):
        if e:
            e.Skip()

        # Make sure that the value is only digits.
        data = self._numKeys[1].GetValue()
        if len(data)>0:
            m = re.search('^[0-9]+$',data)
            if not m:
                self.MyWarning('Invalid character in "%s"' % (data))
                self._numKeys[1].SetValue( ''.join(i for i in data if i.isdigit()) )

    def OnKeyLen(self, e):
        if e:
            e.Skip()

        # Make sure that the value is only digits.
        data = self._keyLen[1].GetValue()
        if len(data)>0:
            m = re.search('^[0-9]+$',data)
            if not m:
                self.MyWarning('Invalid character in "%s"' % (data))
                self._keyLen[1].SetValue( ''.join(i for i in data if i.isdigit()) )
                data = self._keyLen[1].GetValue()
        else:
            data = '0'
        
        # Check for minimum length.
        # Don't do anything here because the user may be selecting
        # and typing.
        newLen = int(data)
        if newLen < 3:
            return

        # Fix up the combo boxes.
        choices = []
        for i in range(newLen+1):
            choices.append( str(i) )

        for i in range(1,3):
            val = min(newLen,int(self._alphaLC[i].GetValue()))
            self._alphaLC[i].Clear()
            self._alphaLC[i].AppendItems(choices)
            self._alphaLC[i].SetValue(str(val))

            val = min(newLen,int(self._alphaUC[i].GetValue()))
            self._alphaUC[i].Clear()
            self._alphaUC[i].AppendItems(choices)
            self._alphaUC[i].SetValue(str(val))

            val = min(newLen,int(self._digits[i].GetValue()))
            self._digits[i].Clear()
            self._digits[i].AppendItems(choices)
            self._digits[i].SetValue(str(val))

            val = min(newLen,int(self._spaces[i].GetValue()))
            self._spaces[i].Clear()
            self._spaces[i].AppendItems(choices)
            self._spaces[i].SetValue(str(val))

            val = min(newLen,int(self._other[i].GetValue()))
            self._other[i].Clear()
            self._other[i].AppendItems(choices)
            self._other[i].SetValue(str(val))

    def MyPopup(self, title, msg):
        """
        Pops up a dialogue with a message.
        Used for internal debugging.
        """
        view = self.GetFirstView()
        topWin = view.GetFrame()
        dlg = wx.MessageDialog(topWin,msg,title,wx.OK)
        dlg.ShowModal()
        dlg.Destroy()

    def OnCbAlphaLC(self, e):
        if e:
            e.Skip()
        if self._alphaLC[0].GetValue():
            self._alphaLC[1].Enable(True)
            self._alphaLC[2].Enable(True)
            self._alphaLC[3].Enable(True)
        else:
            self._alphaLC[1].Enable(False)
            self._alphaLC[2].Enable(False)
            self._alphaLC[3].Enable(False)

    def OnCbAlphaLCMin(self, e):
        if e:
            e.Skip()
        minval = int(self._alphaLC[1].GetValue())
        maxval = int(self._alphaLC[2].GetValue())
        if maxval < minval:
            self._alphaLC[2].SetValue(str(minval))

    def OnCbAlphaLCMax(self, e):
        if e:
            e.Skip()
        minval = int(self._alphaLC[1].GetValue())
        maxval = int(self._alphaLC[2].GetValue())
        if maxval < minval:
            self._alphaLC[1].SetValue(str(maxval))

    def OnCbAlphaUC(self, e):
        if e:
            e.Skip()
        if self._alphaUC[0].GetValue():
            self._alphaUC[1].Enable(True)
            self._alphaUC[2].Enable(True)
            self._alphaUC[3].Enable(True)
        else:
            self._alphaUC[1].Enable(False)
            self._alphaUC[2].Enable(False)
            self._alphaUC[3].Enable(False)

    def OnCbAlphaUCMin(self, e):
        if e:
            e.Skip()
        minval = int(self._alphaUC[1].GetValue())
        maxval = int(self._alphaUC[2].GetValue())
        if maxval < minval:
            self._alphaUC[2].SetValue(str(minval))

    def OnCbAlphaUCMax(self, e):
        if e:
            e.Skip()
        minval = int(self._alphaUC[1].GetValue())
        maxval = int(self._alphaUC[2].GetValue())
        if maxval < minval:
            self._alphaUC[1].SetValue(str(maxval))

    def OnCbDigits(self, e):
        if e:
            e.Skip()
        if self._digits[0].GetValue():
            self._digits[1].Enable(True)
            self._digits[2].Enable(True)
            self._digits[3].Enable(True)
        else:
            self._digits[1].Enable(False)
            self._digits[2].Enable(False)
            self._digits[3].Enable(False)

    def OnCbDigitsMin(self, e):
        if e:
            e.Skip()
        minval = int(self._digits[1].GetValue())
        maxval = int(self._digits[2].GetValue())
        if maxval < minval:
            self._digits[2].SetValue(str(minval))

    def OnCbDigitsMax(self, e):
        if e:
            e.Skip()
        minval = int(self._digits[1].GetValue())
        maxval = int(self._digits[2].GetValue())
        if maxval < minval:
            self._digits[1].SetValue(str(maxval))

    def OnCbSpaces(self, e):
        if e:
            e.Skip()
        if self._spaces[0].GetValue():
            self._spaces[1].Enable(True)
            self._spaces[2].Enable(True)
            self._spaces[3].Enable(True)
        else:
            self._spaces[1].Enable(False)
            self._spaces[2].Enable(False)
            self._spaces[3].Enable(False)

    def OnCbSpacesMin(self, e):
        if e:
            e.Skip()
        minval = int(self._spaces[1].GetValue())
        maxval = int(self._spaces[2].GetValue())
        if maxval < minval:
            self._spaces[2].SetValue(str(minval))

    def OnCbSpacesMax(self, e):
        if e:
            e.Skip()
        minval = int(self._spaces[1].GetValue())
        maxval = int(self._spaces[2].GetValue())
        if maxval < minval:
            self._spaces[1].SetValue(str(maxval))

    def OnCbOther(self, e):
        if e:
            e.Skip()
        if self._other[0].GetValue():
            self._other[1].Enable(True)
            self._other[2].Enable(True)
            self._other[3].Enable(True)
        else:
            self._other[1].Enable(False)
            self._other[2].Enable(False)
            self._other[3].Enable(False)

    def OnCbOtherMin(self, e):
        if e:
            e.Skip()
        minval = int(self._other[1].GetValue())
        maxval = int(self._other[2].GetValue())
        if maxval < minval:
            self._other[2].SetValue(str(minval))

    def OnCbOtherMax(self, e):
        if e:
            e.Skip()
        minval = int(self._other[1].GetValue())
        maxval = int(self._other[2].GetValue())
        if maxval < minval:
            self._other[1].SetValue(str(maxval))

    def OnDone(self, e):
        self.Close()
        self.Destroy()

    def OnHelp(self, e):
        HELP=sys.path[0] + os.sep + 'KeyGeneratorDialog.txt'
        if os.path.exists(HELP):
            fp = open(HELP,'r')
            data = fp.read()
            fp.close()
        else:
            data = 'Help file not found.'
        dialog = DisplayStaticTextDialog(self, 'Key Generator Help')
        dialog.Center()
        dialog.SetValue(data)
        dialog.ShowModal()
        dialog.Destroy()

    def OnCbAlphaLCDup(self, e):
        pass

    def OnCbAlphaUCDup(self, e):
        pass

    def OnCbDigitsDup(self, e):
        pass

    def OnCbSpacesDup(self, e):
        pass

    def OnCbOtherDup(self, e):
        pass
