# ================================================================
# Name:      KeyStrengthDialog.py
# Purpose:   Key strength analyzer dialogue.
# Author:    Joe Linoff
# Created:   2012-01-10
# Id:        $Id: KeyStrengthDialog.py,v 1.5 2012/02/04 01:56:41 jlinoff Exp jlinoff $
# Copyright: (c) 2012 Joe Linoff
# License:
#     SecEd is free software; you can redistribute 
#     it and/or modify it under the terms of the GNU General Public License as 
#     published by the Free Software Foundation; either version 2 of the License, 
#     or (at your option) any later version.
#            
#     SecEd is distributed in the hope that it will be useful, 
#     but WITHOUT ANY WARRANTY; without even the implied warranty of 
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
#     See the GNU General Public License for more details. You should have 
#     received a copy of the GNU General Public License along with SecEd; 
#     if not, write to the Free Software Foundation, Inc., 59 Temple Place, 
#     Suite 330, Boston, MA  02111-1307  USA
# ================================================================

import sys
import os
import wx
import KeyStrength
from DisplayStaticTextDialog import DisplayStaticTextDialog
_ = wx.GetTranslation

class KeyStrengthDialog(wx.Dialog):
    """
    Determine the strength of a user key.
    """

    def __init__(self, parent, id, title, size):
        style = wx.DEFAULT_DIALOG_STYLE | wx.RESIZE_BORDER
        #style = wx.DEFAULT_DIALOG_STYLE
        wx.Dialog.__init__(self, parent, id, title, size=size, style=style)

        # The layout elements.
        vsizer  = wx.BoxSizer(wx.VERTICAL)
        hsizer1 = wx.BoxSizer(wx.HORIZONTAL)
        hsizer2 = wx.BoxSizer(wx.HORIZONTAL)
        hsizer3 = wx.BoxSizer(wx.HORIZONTAL)
        hsizer4 = wx.BoxSizer(wx.HORIZONTAL)
        hsizer5 = wx.BoxSizer(wx.HORIZONTAL)
        hsizer6 = wx.BoxSizer(wx.HORIZONTAL)

        self.BORDER=5

        # Key
        # NOTE: I need to use two controls here because wxPython does not seem allow
        #       changing this style attribute (at least on Windows).
        #       One of the controls is a plaintext TextCtrl and other is a password
        #       TextCtrl. At any given time only one of them is visible.
        key=""
        self._keyCtrl  = wx.TextCtrl(self, -1, key, size=(320,-1))
        self._keyCtrlX = wx.TextCtrl(self, -1, key, size=(320,-1), style=wx.TE_PASSWORD)
        label1 = wx.StaticText(self,-1,_("Key:"),style=wx.ALIGN_RIGHT)
        hsizer1.Add(label1, 0, wx.ALIGN_CENTER_VERTICAL|wx.LEFT|wx.EXPAND, self.BORDER)
        hsizer1.Add(self._keyCtrl, 0, wx.ALIGN_CENTER_VERTICAL|wx.LEFT|wx.EXPAND, self.BORDER)
        hsizer1.Add(self._keyCtrlX,0, wx.ALIGN_CENTER_VERTICAL|wx.LEFT|wx.EXPAND, self.BORDER)
        self._keyCtrl.Hide()
        self._keyCtrl.Bind(wx.EVT_KEY_UP, self.OnKeyCtrl)
        self._keyCtrlX.Bind(wx.EVT_KEY_UP, self.OnKeyCtrlX)
        font10 = wx.Font(10, wx.DEFAULT, wx.NORMAL, wx.NORMAL, 0)
        self._keyCtrl.SetFont(font10)
        self._keyCtrlX.SetFont(font10)
        label1.SetFont(font10)

        # Attempts per second
        aps="10000"
        self._apsCtrl  = wx.TextCtrl(self, -1, aps, size=(320,-1))
        label2 = wx.StaticText(self,-1,_("APS:"),style=wx.ALIGN_RIGHT)
        hsizer2.Add(label2, 0, wx.ALIGN_CENTER_VERTICAL|wx.LEFT|wx.EXPAND, self.BORDER)
        hsizer2.Add(self._apsCtrl, 0, wx.ALIGN_CENTER_VERTICAL|wx.LEFT|wx.EXPAND, self.BORDER)
        self._apsCtrl.Bind(wx.EVT_KEY_UP, self.OnApsCtrl)
        self._apsCtrl.SetFont(font10)
        label2.SetFont(font10)

        # Character categories
        spacer = '    '
        sbox = wx.StaticBox(self, wx.ID_ANY, 'Key Character Categories')
        sbox.SetFont(font10)
        sboxsizer = wx.StaticBoxSizer(sbox, wx.HORIZONTAL)
        self._cbDigits  = wx.CheckBox(self,-1,'Digits'    )
        self._cbAlphaLC = wx.CheckBox(self,-1,'Lower Case')
        self._cbAlphaUC = wx.CheckBox(self,-1,'Upper Case')
        self._cbWS      = wx.CheckBox(self,-1,'Whitespace')
        self._cbOther   = wx.CheckBox(self,-1,'Other'     )

        self._cbDigits.SetFont(font10)
        self._cbAlphaLC.SetFont(font10)
        self._cbAlphaUC.SetFont(font10)
        self._cbWS.SetFont(font10)
        self._cbOther.SetFont(font10)

        self.Bind(wx.EVT_CHECKBOX, self.OnCheckBox, self._cbDigits)
        self.Bind(wx.EVT_CHECKBOX, self.OnCheckBox, self._cbAlphaLC)
        self.Bind(wx.EVT_CHECKBOX, self.OnCheckBox, self._cbAlphaUC)
        self.Bind(wx.EVT_CHECKBOX, self.OnCheckBox, self._cbWS)
        self.Bind(wx.EVT_CHECKBOX, self.OnCheckBox, self._cbOther)

        sboxsizer.Add(self._cbDigits  , 0)
        sboxsizer.Add(wx.StaticText(self,-1,spacer))
        sboxsizer.Add(self._cbAlphaLC , 0)
        sboxsizer.Add(wx.StaticText(self,-1,spacer))
        sboxsizer.Add(self._cbAlphaUC , 0)
        sboxsizer.Add(wx.StaticText(self,-1,spacer))
        sboxsizer.Add(self._cbWS      , 0)
        sboxsizer.Add(wx.StaticText(self,-1,spacer))
        sboxsizer.Add(self._cbOther   , 0)

        hsizer5.Add(sboxsizer , 1, wx.ALIGN_CENTER_VERTICAL|wx.LEFT|wx.EXPAND, 2*self.BORDER)

        # Strength
        self._rbStrengthSel = 0
        self._rbStrength = wx.RadioBox(self,
                                       1,
                                       'Strength Ratings',
                                       choices = ['Super Weak',
                                                  'Very Weak',
                                                  'Weak',
                                                  'Moderate',
                                                  'Strong',
                                                  'Very Strong',
                                                  'Super Strong'],
                                       majorDimension=3,
                                       style = wx.RA_SPECIFY_COLS)
        font12 = wx.Font(12, wx.DEFAULT, wx.NORMAL, wx.NORMAL, 0)
        self._rbStrength.SetFont(font12)

        hsizer6.Add(self._rbStrength, 0, wx.ALIGN_CENTER_VERTICAL|wx.LEFT|wx.EXPAND, self.BORDER)
        self.Bind(wx.EVT_RADIOBOX, self.OnRbStrength, self._rbStrength)

        # Status area
        self._text = wx.TextCtrl(self, -1, style=wx.ALIGN_LEFT|wx.TE_MULTILINE|wx.TE_READONLY)
        hsizer3.Add(self._text, 1, wx.ALIGN_CENTER_VERTICAL|wx.LEFT|wx.EXPAND|wx.ALL, self.BORDER)
        font = wx.Font(12, wx.DEFAULT, wx.NORMAL, wx.NORMAL, 0, 'Courier')
        self._text.SetFont(font)
        self.Analyze('')
        self._text.SetInitialSize((750, 200))

        # Buttons
        okBtn     = wx.Button(self, wx.ID_ANY, 'OK')
        showBtn   = wx.Button(self, wx.ID_ANY, 'Show')  # Show/Hide
        helpBtn   = wx.Button(self, wx.ID_ANY, 'Help')

        hsizer4.Add(okBtn    , 0, wx.ALL, self.BORDER)
        hsizer4.Add(showBtn  , 0, wx.ALL, self.BORDER)
        hsizer4.Add(helpBtn  , 0, wx.ALL, self.BORDER)

        self.Bind(wx.EVT_BUTTON, self.OnOk     , okBtn)
        self.Bind(wx.EVT_BUTTON, self.OnShow   , showBtn)
        self.Bind(wx.EVT_BUTTON, self.OnHelp   , helpBtn)

        # Layout
        vsizer.Add(hsizer1, 0, wx.LEFT|wx.RIGHT|wx.BOTTOM|wx.TOP|wx.EXPAND, 2*self.BORDER)
        vsizer.Add(hsizer2, 0, wx.LEFT|wx.RIGHT|wx.BOTTOM|wx.EXPAND, 2*self.BORDER)
        vsizer.Add(hsizer6, 0, wx.LEFT|wx.RIGHT|wx.BOTTOM|wx.EXPAND, 2*self.BORDER)
        vsizer.Add(hsizer5, 0, wx.LEFT|wx.RIGHT|wx.BOTTOM|wx.EXPAND, 2*self.BORDER)
        vsizer.Add(hsizer3, 1, wx.LEFT|wx.RIGHT|wx.BOTTOM|wx.EXPAND, 2*self.BORDER)
        vsizer.Add(hsizer4, 0, wx.LEFT|wx.RIGHT|wx.BOTTOM|wx.EXPAND, 2*self.BORDER)

        self.Layout()
        self.SetSizer(vsizer)
        self.Fit()
        self._keyCtrlX.SetFocus()

        self._showBtn = showBtn
        self._showPlaintext = False

        # Now set the minimum based on the fit operation.
        size = self.GetSize()
        self.SetSizeHints( minW=size.GetWidth(), minH=size.GetHeight() )

    def OnShow(self, e):
        if self._showPlaintext == False:
            # Show the plaintext controls.
            self._showPlaintext = True

            # Sync values
            self._keyCtrl.SetValue( self._keyCtrlX.GetValue() )

            # Set display
            self._keyCtrl.Show()
            self._keyCtrlX.Hide()

            # Change the button label
            self._showBtn.SetLabel('Hide')
        else:
            # Show the password controls.
            self._showPlaintext = False

            # Sync values
            self._keyCtrlX.SetValue( self._keyCtrl.GetValue() )

            # Set display
            self._keyCtrl.Hide()
            self._keyCtrlX.Show()

            # Change the button label
            self._showBtn.SetLabel('Show') # change the button label

        self.Layout()
        self.Fit()
        self.Refresh()

    def OnKeyCtrl(self, e):
        data = self._keyCtrl.GetValue()
        self._keyCtrlX.SetValue(data) # keep them sync'ed
        self.Analyze(data)
        e.Skip()

    def OnKeyCtrlX(self, e):
        data = self._keyCtrlX.GetValue()
        self._keyCtrl.SetValue(data) # keep them sync'ed
        self.Analyze(data)
        e.Skip()

    def OnApsCtrl(self, e):
        # When the user enters an APS value, recalculate for
        # the key.
        # keyCtrl and keyCtrlX are synced.
        data = self._keyCtrl.GetValue()
        self.Analyze(data)
        e.Skip()

    def OnCheckBox(self, e):
        if e:
            e.StopPropagation()
            e.Skip()
        # Don't let the user change it interactively.
        self._cbDigits.SetValue (self._cbDigitsValue)
        self._cbAlphaLC.SetValue(self._cbAlphaLCValue)
        self._cbAlphaUC.SetValue(self._cbAlphaUCValue)
        self._cbWS.SetValue     (self._cbWSValue)
        self._cbOther.SetValue  (self._cbOtherValue)

    def OnRbStrength(self, e):
        if e:
            e.StopPropagation()
            e.Skip()
        # Don't let the user change it interactively.
        self._rbStrength.SetSelection(self._rbStrengthSel)

    def Analyze(self, key):
        aps = float( self._apsCtrl.GetValue() )
        keylen = len(key)
        ks = KeyStrength.KeyStrength(key,aps)
        keylen = len(key)
        cat, base, comb, maxttb = ks.GetCombinations()
        yrs, days, hrs, mins, secs = ks.GetTimeInfo(maxttb)
        rating, desc = ks.GetRating(cat, maxttb)

        if maxttb>=2:
            ttb_str = ks.Commaize(maxttb)+' secs'
        elif maxttb==12:
            ttb_str = '1 sec'
        else:
            ttb_str = 'less than 1 sec'

        if yrs>=1000:
            et = 'more than a millenium: %s years' % (ks.Commaize(yrs))
        elif yrs>=1:
            et = '%d years, %d days, %02d:%02d:%02d' % (yrs,days,hrs,mins,secs)
        elif days>=1:
            et = '%d days, %02d:%02d:%02d' % (days,hrs,mins,secs)
        elif yrs==0 and mins == 0 and secs == 0:
            et = ttb_str
        elif yrs==0 and mins == 0 and secs == 1:
            et = ttb_str
        else:
            et = '%02d:%02d:%02d' % (hrs,mins,secs)

        cat_tuple = '%d, %d, %d, %d, %d' % (cat['charDigits'][0],
                                            cat['charAlphaLC'][0],
                                            cat['charAlphaUC'][0],
                                            cat['charWS'][0],
                                            cat['charOther'][0])
        data = ''
        data += '%-17s : %s\n' % ('Rating'           ,rating)
        data += '%-17s : %s\n' % ('Strength'         ,desc)
        data += '%-17s : %s\n' % ('Combinations'     ,ks.Commaize(comb))
        data += '%-17s : %s\n' % ('Analysis Time'    ,et)
        data += '%-17s : %s\n' % ('Analysis Time Raw',ttb_str)
        data += '%-17s : %d\n' % ('Key Length'       ,keylen)
        data += '%-17s : %d\n' % ('Base'             ,base)
        data += '%-17s : %s\n' % ('Category Tuple'   ,cat_tuple)

        self._text.SetValue(data.rstrip())

        self._cbDigitsValue  = cat['charDigits'][0] > 0
        self._cbAlphaLCValue = cat['charAlphaLC'][0] > 0
        self._cbAlphaUCValue = cat['charAlphaUC'][0] > 0
        self._cbWSValue      = cat['charWS'][0] > 0
        self._cbOtherValue   = cat['charOther'][0] > 0

        self.OnCheckBox(None)

        if rating == 'Super Weak':
            self._rbStrengthSel = 0
        elif rating == 'Very Weak':
            self._rbStrengthSel = 1
        elif rating == 'Weak':
            self._rbStrengthSel = 2
        elif rating == 'Moderate':
            self._rbStrengthSel = 3
        elif rating == 'Strong':
            self._rbStrengthSel = 4
        elif rating == 'Very Strong':
            self._rbStrengthSel = 5
        elif rating == 'Super Strong':
            self._rbStrengthSel = 6

        self.OnRbStrength(None)

    def OnOk(self, e):
        self.Close()
        self.Destroy()

    def OnHelp(self, e):
        HELP=sys.path[0] + os.sep + 'KeyStrengthDialog.txt'
        if os.path.exists(HELP):
            fp = open(HELP,'r')
            data = fp.read()
            fp.close()
        else:
            data = 'Help file not found.'
        dialog = DisplayStaticTextDialog(self, 'Key Strength Help')
        dialog.Center()
        dialog.SetValue(data)
        dialog.ShowModal()
        dialog.Destroy()
