# ================================================================
# Name:      SecureData.py
# Purpose:   Encrypt and/or decrypt data.
# Author:    Joe Linoff
# Created:   2012-01-10
# Id:        $Id: SecureData.py,v 1.2 2012/01/13 02:54:18 jlinoff Exp jlinoff $
# Copyright: (c) 2012 Joe Linoff
# License:
#     SecEd is free software; you can redistribute 
#     it and/or modify it under the terms of the GNU General Public License as 
#     published by the Free Software Foundation; either version 2 of the License, 
#     or (at your option) any later version.
#            
#     SecEd is distributed in the hope that it will be useful, 
#     but WITHOUT ANY WARRANTY; without even the implied warranty of 
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
#     See the GNU General Public License for more details. You should have 
#     received a copy of the GNU General Public License along with SecEd; 
#     if not, write to the Free Software Foundation, Inc., 59 Temple Place, 
#     Suite 330, Boston, MA  02111-1307  USA
# ================================================================

import os
import sys
import bz2
import base64

sys.path.append(sys.path[0]+'/pyrijndael')
import pyRijndael

class SecureData:
    """
    Decrypt/encrypt data.
    """
    def __init__(self):
        pass

    def Decrypt(self, key, m64_data):
        """
        Decrypt data.

        Example:
           sd = SecureData()
           ok,txt = sd.Decrypt(key,data)
           if not ok:
               print 'ERROR: invalid key'
        
        Returns:
           The status and the decrypted data.
           If the status is False, it means that the key was bad.
        """
        ok = True;
        cpt_data = base64.b64decode(m64_data)
        bz2_data = pyRijndael.DecryptData(key,cpt_data)
        try:
            txt_data = bz2.decompress(bz2_data)

            # Do a final sanity check.
            if len(m64_data)>0 and len(txt_data) == 0:
                ok = False
        except IOError:
            txt_data = ''
            ok = False
        return ok,txt_data

    def Encrypt(self, key, txt_data):
        """
        Encrypt data.

        Returns the MIME 64 encoded data.
        """
        bz2_data = bz2.compress(txt_data)
        cpt_data = pyRijndael.EncryptData(key,bz2_data)
        m64_data = base64.b64encode(cpt_data)
        return m64_data
