#!/usr/bin/env python2.7
import getpass
import getopt
import os
import sys
import re
import datetime
import shlex, subprocess
import bz2
import base64
import tempfile
import time
import SecureData

def usage():
    """
    Print the program help and exit.
    """
    fn = os.path.basename(__file__)
    print """
USAGE
    %s [OPTIONS] <input-file> [<output-file>]

DESCRIPTION
    Descrypt a file that was encrypted using seced-encrypt.py.

OPTIONS
    There are short and long forms for most options.

    -k <key>, --key <key>
                 Explicitly specify the decryption key on the
                 command line.

    -h, --help   Display this usage message.

    -v, --verbose
                 The level of verbosity. It can be specified
                 multiple times.

    -V, --version
                 Print the program version and exit.

EXAMPLES
    %% %s -h

    %% # Convert a file from stdout.
    %% %s mydata.txt > mysecuredata.txt
    Enter decryption key:
    Enter decryption key: (repeat)

    %% # Convert a file with the output explicitly 
    %% # specified.
    %% %s mydata.txt mysecuredata.txt
    Enter decryption key:
    Enter decryption key: (repeat)

    %% # Convert a file with the output explicitly 
    %% # specified and the key specified on the 
    %% # command line.
    %% %s -k 'mykey123' mydata.txt mysecuredata.txt

AUTHOR
    Joe Linoff (jlinoff@tabula.com)

VERSION
    $Id: seced-decrypt.py,v 1.2 2012/01/10 06:03:55 jlinoff Exp $

COPYRIGHT
    %s allows you to decrypt files.

    Copyright (c) 2011 by Joe Linoff

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    """ % (fn,fn,fn,fn,fn,fn)
    sys.exit(0)

def main():
    try:
        opts, args = getopt.gnu_getopt(sys.argv[1:],
                                       "hk:vV",
                                       ["help",
                                        "key=",
                                        "verbose",
                                        "version"])
    except getopt.GetoptError, err:
        print str(err)
        print "exiting ..."
        sys.exit(1)

    verbose=0
    key=None
    for opt,arg in opts:
        if opt in ["-h","--help"] :
            usage()
        elif opt in ["-k","--key"] :
            key=arg
        elif opt in ["-v","--verbose"] :
            verbose += 1
        elif opt in ["-V","--version"] :
            print 'Version: $Id: seced-decrypt.py,v 1.2 2012/01/10 06:03:55 jlinoff Exp $'
            sys.exit(0)
        else:
            print "ERROR: Unrecognized option "+opt
            sys.exit(1)
    
    fns = []
    for fn in args:
        fns.append(fn)

    if len(fns) > 2:
        print "ERROR: too many files specified."
        sys.exit(1)

    if (len(fns)) == 0:
        print "ERROR: no input files specified"
        sys.exit(1)

    if not key:
        key=getpass.getpass('Enter decryption key: ')

    # ================================================================
    # Read the file
    # ================================================================
    if len(fns)>0:
        if verbose:
            print "reading file %s ..." % (fns[0])
        try:
            fp = open(fns[0],'rb')
            data = fp.read()
            fp.close()
        except:
            print "ERROR: read failed for %s" % (fns[0])
            sys.exit(1)

    # ================================================================
    # Encrypt the data
    # ================================================================
    if verbose:
        print "read %d bytes" % (len(data))
        print "decrypting data ..."
    sd = SecureData.SecureData()
    ok, dec_data = sd.Decrypt(key, data)
    if not ok:
        print "ERROR: decryption failed, the key was incorrect."
        sys.exit(1)

    # ================================================================
    # Write the file
    # ================================================================
    if verbose:
        print "decrypted %d bytes" % (len(dec_data))
    if len(fns)>1:
        if verbose:
            print "writing data to %s ..." % (fns[1])
        try:
            fp = open(fns[1],'wb')
            fp.write(dec_data)
            fp.close()
            if verbose:
                print "wrote %d bytes to %s " % (os.path.getsize(fns[1]),fns[1])
        except:
            print "ERROR: write failed for %s" % (fns[1])
            sys.exit(1)
    else:
        print dec_data

if __name__ == "__main__":
    main()
