#!/usr/bin/env python2.7
import getpass
import getopt
import os
import sys
import re
import datetime
import shlex, subprocess
import bz2
import base64
import tempfile
import time
import SecureData

def usage():
    """
    Print the program help and exit.
    """
    fn = os.path.basename(__file__)
    print """
USAGE
    %s [OPTIONS] <input-file> [<output-file>]

DESCRIPTION
    Encrypt a file using the AES Rijndael Block Cipher algorithm
    provided by Jeffrey Clement. The output is MIME 64 encoded to
    make it convenient to copy between platforms.

OPTIONS
    There are short and long forms for most options.

    -k <key>, --key <key>
                 Explicitly specify the decryption key on the
                 command line.

    -h, --help   Display this usage message.

    -v, --verbose
                 The level of verbosity. It can be specified
                 multiple times.

    -V, --version
                 Print the program version and exit.

EXAMPLES
    %% %s -h

    %% # Convert a file from stdout.
    %% %s mydata.txt > mysecuredata.txt
    Enter encryption key:
    Enter encryption key: (repeat)

    %% # Convert a file with the output explicitly 
    %% # specified.
    %% %s mydata.txt mysecuredata.txt
    Enter encryption key:
    Enter encryption key: (repeat)

    %% # Convert a file with the output explicitly 
    %% # specified and the key specified on the 
    %% # command line.
    %% %s -k 'mykey123' mydata.txt mysecuredata.txt

AUTHOR
    Joe Linoff (jlinoff@tabula.com)

VERSION
    $Id: seced-encrypt.py,v 1.2 2012/01/10 06:03:55 jlinoff Exp $

COPYRIGHT
    %s allows you to encrypt files.

    Copyright (c) 2011 by Joe Linoff

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    """ % (fn,fn,fn,fn,fn,fn)
    sys.exit(0)

def main():
    try:
        opts, args = getopt.gnu_getopt(sys.argv[1:],
                                       "hk:vV",
                                       ["help",
                                        "key=",
                                        "verbose",
                                        "version"])
    except getopt.GetoptError, err:
        print str(err)
        print "exiting ..."
        sys.exit(1)

    verbose=0
    key=None
    for opt,arg in opts:
        if opt in ["-h","--help"] :
            usage()
        elif opt in ["-k","--key"] :
            key=arg
        elif opt in ["-v","--verbose"] :
            verbose += 1
        elif opt in ["-V","--version"] :
            print 'Version: $Id: seced-encrypt.py,v 1.2 2012/01/10 06:03:55 jlinoff Exp $'
            sys.exit(0)
        else:
            print "ERROR: Unrecognized option "+opt
            sys.exit(1)
    
    fns = []
    for fn in args:
        fns.append(fn)

    if len(fns) > 2:
        print "ERROR: too many files specified."
        sys.exit(1)

    if not key:
        max_tries=3
        n=0
        key1='1'
        key2='2'
        while key1 != key2:
            n += 1
            if n>max_tries:
                print "ERROR: mismatched keys, no more tries."
                sys.exit(1)
            elif n>1:
                print "ERROR: mismatched keys, please try again."
                print
            key1=getpass.getpass('Enter encryption key: ')
            key2=getpass.getpass('Enter encryption key: (repeat) ')
        key=key1

    # ================================================================
    # Read the file
    # ================================================================
    if len(fns)>0:
        if verbose:
            print "reading file %s ..." % (fns[0])
        try:
            fp = open(fns[0],'rb')
            data = fp.read()
            fp.close()
        except:
            print "ERROR: read failed for %s" % (fns[0])
            sys.exit(1)
    else:
        print 'Enter plaintext data? ',
        data = sys.stdin.readline()

    # ================================================================
    # Encrypt the data
    # ================================================================
    if verbose:
        print "read %d bytes" % (len(data))
        print "encrypting data ..."
    sd = SecureData.SecureData()
    enc_data = sd.Encrypt(key, data)

    # ================================================================
    # Write the file
    # ================================================================
    if verbose:
        print "encrypted %d bytes" % (len(enc_data))
    if len(fns)>1:
        if verbose:
            print "writing data to %s ..." % (fns[1])
        try:
            fp = open(fns[1],'wb')
            fp.write(enc_data)
            fp.close()
            if verbose:
                print "wrote %d bytes to %s " % (os.path.getsize(fns[1]),fns[1])
        except:
            print "ERROR: write failed for %s" % (fns[1])
            sys.exit(1)
    else:
        print enc_data

if __name__ == "__main__":
    main()
